xquery version "3.1";

module namespace contentapi="http://sepia.io/content-parser/content-api";

declare namespace http="http://expath.org/ns/http-client";
declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace cpcf="http://sepia.io/content-parser/ns/configfile";

declare variable $contentapi:file := doc("config.xml");

(:~
 : A function to query the config for module specific parameter.
 : @param $key – the name of the parameter to query for
 : @param $module-key – the name of the module
 :  :)
declare function contentapi:getConfig($key as xs:string, $module-key as xs:string?) {
    let $base := $contentapi:file/cpcf:config/cpcf:module[@key = $module-key]/cpcf:param[@key = $key]
    let $elements := $base/*
    return if($elements) then $elements else $base/node()
};

declare function contentapi:content($node as node(), $model as map(*), $id as xs:string)
{
    let $contentapiMultilangEnabled := contentapi:getConfig("contentapi.multilang.enabled", "contentapi")
    let $contentapiBranch := contentapi:getConfig("contentapi.branch", "contentapi")
    let $lang :=
        if ($contentapiMultilangEnabled eq "true") then (
            let $configLang := contentapi:getLanguage()
            return "_" || $configLang
        )
        else ""  
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")
    let $contentapiRepo := contentapi:getConfig("contentapi.repo", "contentapi")
    let $id := encode-for-uri($id)
    let $requestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapiRepo || "/repository/files/" || $id || $lang || ".md?ref=" || $contentapiBranch
    let $version := contentapi:check-version($id, $lang)
    let $document :=
        if ($version != "none") then
            doc($contentapiTarget || "/" || $id || $lang || '.hash' || $version || '.html' )
        else
            contentapi:update-document($id, $lang)
            
    (: check for image updates: true if there is an update available, then get all images and store them :)
    let $imageCheck := 
        if (local:checkForImageUpdates()) then (
            local:storeImageList(),
            contentapi:updateImages()
        )
        else ("no image updates available")

    return
        $document
};

declare function contentapi:update-document($id, $lang)
{
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")
    let $contentapiSecret := contentapi:getConfig("contentapi.secret", "contentapi")
    let $contentapiBranch := contentapi:getConfig("contentapi.branch", "contentapi")
    let $markdownRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapiSecret}' /></http:request>
    let $markdownRequestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || contentapi:getConfig("contentapi.repo", "contentapi") || "/repository/files/" || $id || $lang || ".md?ref=" || $contentapiBranch
    let $rest := http:send-request($markdownRequest, $markdownRequestUrl)
    let $version := $rest[1]//*[@name="x-gitlab-last-commit-id"]/@value
    let $markdown := http:send-request($markdownRequest, $markdownRequestUrl)[2]
    let $markdown := $markdown => util:base64-decode() => parse-json() => map:get("content") => util:base64-decode()

    let $transformRequest := <http:request method="post"><http:header name="PRIVATE-TOKEN" value='{$contentapiSecret}' /><http:body media-type="application/json" method="text"/></http:request>
    let $transformRequestUrl := "https://gitlab.gwdg.de/api/v4/markdown"
    let $payload := serialize(map{"text": $markdown}, <output:serialization-parameters><output:method>json</output:method></output:serialization-parameters>)
    let $responseBody := http:send-request($transformRequest, $transformRequestUrl, $payload)[2]

    let $content := $responseBody => util:base64-decode() => parse-json() => map:get("html") => util:parse-html()
    let $login := xmldb:login("/db", "admin", "" )
    let $save := if ($content) then xmldb:store($contentapiTarget, $id || $lang || ".hash" || $version || ".html", $content) else ()

    return
        $content
};

declare function contentapi:check-version($id as xs:string, $lang as xs:string)
{
    let $local := contentapi:get-current-version($id, $lang)
    let $remote := contentapi:get-remote-version($id, $lang)

    return
        if ($local = $remote and $local != "none") then
            $local
        else
            "none"
};

declare function contentapi:get-current-version($id as xs:string, $lang as xs:string)
{
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")
    let $version := xmldb:get-child-resources($contentapiTarget)[contains(., $id) and contains(., $lang)][1]
        => substring-after(".hash")
        => substring-before(".html")

    return
        if ($version) then
            $version
        else
            "none"
};

declare function contentapi:get-remote-version($id as xs:string, $lang as xs:string)
{
    let $contentapiSecret := contentapi:getConfig("contentapi.secret", "contentapi")
    let $contentapiBranch := contentapi:getConfig("contentapi.branch", "contentapi")
    let $httpRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapiSecret}' /></http:request>
    let $requestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || contentapi:getConfig("contentapi.repo", "contentapi") || "/repository/files/" || $id || $lang || ".md?ref=" || $contentapiBranch
    let $rest := http:send-request($httpRequest, $requestUrl)

    return
        if ($rest[1]/@status = 200) then
            $rest[1]//*[@name="x-gitlab-last-commit-id"]/@value
        else
            "none"
};

declare function contentapi:getLanguage(){
    let $langDefault := contentapi:getConfig("contentapi.multilang.default", "contentapi")
    let $langAlt := contentapi:getConfig("contentapi.multilang.alt", "contentapi")
    let $langGetParameter := request:get-parameter("lang","")
    let $langBrowser := fn:substring(request:get-header("Accept-Language"),0,3)
    let $langInput := 
        if ($langGetParameter != "") then $langGetParameter
        else $langBrowser
    let $langOutput := 
        if ($langInput = "") then $langDefault
        else if ((fn:contains($langInput,$langDefault)) or (fn:contains($langInput,$langAlt))) then $langInput
        else $langDefault
    return $langOutput
};

declare function local:checkForImageUpdates() as xs:boolean {
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")

    let $local-doc-name := $contentapiTarget || "/images.json"
    let $localList :=
        if (fn:exists($local-doc-name)) then
            util:binary-doc($local-doc-name) => util:base64-decode()
        else "none"
    let $remoteList := local:getImageList()

    return
        if ($localList != "none" and $localList = $remoteList) then
            false()
        else
            true()
};

declare function local:storeImageList() {
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")
    let $login := xmldb:login("/db", "admin", "" )
    let $listOfAllImages := local:getImageList()
    let $store := xmldb:store($contentapiTarget,"images.json",$listOfAllImages,"application/json")
    return "done"
};

declare function local:getImageList() {
    let $contentapiRepo := contentapi:getConfig("contentapi.repo", "contentapi")
    let $contentapiBranch := contentapi:getConfig("contentapi.branch", "contentapi")
    let $contentapiSecret := contentapi:getConfig("contentapi.secret", "contentapi")
    
    let $httpRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapiSecret}' /></http:request>
    let $requestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapiRepo || "/repository/tree?ref=" || $contentapiBranch || "&amp;path=img&amp;per_page=1000"

    let $rest := http:send-request($httpRequest, $requestUrl)
    let $listOfAllImages :=
        if ($rest[1]/@status = 200) then $rest[2] => util:base64-decode()
        else (QName("http://sepia.io/content-parser/content-api", "CONTENTPARSER11"), "Could not access GitLabAPI to get images: " || $rest[1]/@status)
    return $listOfAllImages
};

declare function contentapi:updateImages(){
    let $contentapiRepo := contentapi:getConfig("contentapi.repo", "contentapi")
    let $contentapiSecret := contentapi:getConfig("contentapi.secret", "contentapi")
    let $contentapiTarget := contentapi:getConfig("contentapi.target", "contentapi")
    let $contentapiBranch := contentapi:getConfig("contentapi.branch", "contentapi")

    let $listOfAllImages := local:getImageList() => parse-json()
    let $contentapiTargetImages := $contentapiTarget || "/img/"

    let $httpRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapiSecret}' /></http:request>    
    let $requestUrlImage := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapiRepo || "/repository/files/img%2F"
    let $login := xmldb:login("/db", "admin", "")
    let $create-dir :=
        if (xmldb:collection-available($contentapiTarget || "/img")) then ()
        else (xmldb:create-collection($contentapiTarget,"img"))
    
    for $image in 1 to array:size($listOfAllImages) 
        let $imageName := map:get($listOfAllImages($image),"name")
        let $imageId := map:get($listOfAllImages($image),"id")
        let $rest := http:send-request($httpRequest, $requestUrlImage || $imageName || "?ref=main")
        let $imageBinary := $rest[2] => util:base64-decode() => parse-json() => map:get("content") => xs:base64Binary()
        let $save := xmldb:store($contentapiTarget || "/img", $imageName, $imageBinary)
    return "done"
};